package net.esper.eql.join.exec;

import net.esper.event.EventBean;
import net.esper.event.EventType;
import net.esper.event.EventPropertyGetter;
import net.esper.event.EventBeanUtility;
import net.esper.eql.join.table.PropertyIndexedEventTable;

import java.util.Set;
import java.util.Arrays;

/**
 * Lookup on an index using a set of properties as key values.
 */
public class IndexedTableLookupStrategy implements TableLookupStrategy
{
    private final EventType eventType;
    private final String[] properties;
    private final PropertyIndexedEventTable index;
    private final EventPropertyGetter[] propertyGetters;

    /**
     * Ctor.
     * @param eventType - event type to expect for lookup
     * @param properties - key properties
     * @param index - index to look up in
     */
    public IndexedTableLookupStrategy(EventType eventType, String[] properties, PropertyIndexedEventTable index)
    {
        this.eventType = eventType;
        this.properties = properties;
        this.index = index;

        propertyGetters = new EventPropertyGetter[properties.length];
        for (int i = 0; i < properties.length; i++)
        {
            propertyGetters[i] = eventType.getGetter(properties[i]);

            if (propertyGetters[i] == null)
            {
                throw new IllegalArgumentException("Property named '" + properties[i] + "' is invalid for type " + eventType);
            }
        }
    }

    /**
     * Returns event type of the lookup event.
     * @return event type of the lookup event
     */
    public EventType getEventType()
    {
        return eventType;
    }

    /**
     * Returns properties to use from lookup event to look up in index.
     * @return properties to use from lookup event
     */
    public String[] getProperties()
    {
        return properties;
    }

    /**
     * Returns index to look up in.
     * @return index to use
     */
    public PropertyIndexedEventTable getIndex()
    {
        return index;
    }

    public Set<EventBean> lookup(EventBean event)
    {
        Object[] keys = getKeys(event);
        return index.lookup(keys);
    }

    private Object[] getKeys(EventBean event)
    {
        return EventBeanUtility.getPropertyArray(event, propertyGetters);
    }

    public String toString()
    {
        return "IndexedTableLookupStrategy indexProps=" + Arrays.toString(properties) +
                " index=(" + index + ")";
    }
}
